import { defineEventHandler, getQuery, getMethod, readBody } from "h3";
import {
  queryParamToVariables,
  getEndpoint,
  getFetchOptions,
  validateRequest,
  onServerResponse,
  onServerError
} from "./helpers";
import { GraphqlMiddlewareOperation } from "./../settings/index.mjs";
import { documents } from "#graphql-documents";
import { useRuntimeConfig } from "#imports";
import serverOptions from "#graphql-middleware-server-options-build";
export default defineEventHandler(async (event) => {
  const method = getMethod(event);
  const operation = event.context.params.operation;
  const name = event.context.params.name;
  validateRequest(method, operation, name, documents);
  const query = documents[operation][name];
  const runtimeConfig = useRuntimeConfig().graphqlMiddleware;
  const endpoint = await getEndpoint(
    runtimeConfig,
    serverOptions,
    event,
    operation,
    name
  );
  const fetchOptions = await getFetchOptions(
    serverOptions,
    event,
    operation,
    name
  );
  const variables = operation === GraphqlMiddlewareOperation.Query ? queryParamToVariables(getQuery(event)) : await readBody(event);
  return $fetch.raw(endpoint, {
    // @todo: Remove any once https://github.com/unjs/nitro/pull/883 is released.
    method: "POST",
    body: {
      query,
      variables,
      operationName: name
    },
    ...fetchOptions
  }).then((response) => {
    return onServerResponse(serverOptions, event, response, operation, name);
  }).catch((error) => {
    return onServerError(serverOptions, event, error, operation, name);
  });
});
